#!/usr/bin/env python3

# FIXME: Some projects are currently modeled by creating several jenkaas
# "projects" (-charm, -dependencies). Ideally there should be a single jenkaas
# project file defining all the jobs. -- vila 2016-10-04

import collections
import os
import sys


root_dir = os.path.dirname(os.path.dirname(__file__))
jobdefs_path = os.path.expanduser('~/u1/ubuntuone-infrastructure/puppet/'
                                  'manifests/classes/u1-jenkins.pp')


def collect_jobs():
    jobs = collections.OrderedDict()
    job_name = None
    job_def = dict()
    with open(jobdefs_path) as jobdefs:
        start_seen = False
        for line in jobdefs.readlines():
            if 'include jenkins::tarmac::trigger' in line:
                start_seen = True
                continue
            if not start_seen:
                continue
            if not line.strip():  # Skip empty/blank lines
                continue
            if line.lstrip().startswith('#'):  # Skip comments
                continue
            line = line.rstrip()
            if line == '}':
                break  # We're done
            if 'jenkins::' in line:
                # jenkins::tarmac::job { "lp:python-oops-dictconfig":
                args = line.split()
                job_def['type'] = args[0]
                # "lp:python-oops-dictconfig":
                job_name = args[2][1:-2]
                continue
            if '=>' in line:
                # tree_dir => "isitdeployable/merges/trunk",
                args = line.split('=>')
                job_def[args[0].strip()] = args[1].strip()[1:-2]
                continue
            if line.endswith('}'):
                # Job def ends
                jobs[job_name] = job_def
                job_def = dict()
                job_name = None
                continue
            # Display unrecognized lines for debug
            print('Not recognized: {}'.format(line))
    return jobs


jenkaas_job_dir = os.path.join(root_dir, 'jobs')
projects = collections.OrderedDict()


def project_name_from_puppet_name(name):
    if name.startswith('lp:~'):
        _, project_name, branch_name = name.split('/')
        # But there are exceptions
        if project_name == 'canonical-mojo-specs':
            project_name += '-' + branch_name
        elif project_name == 'canonical-is-charms':
            project_name = branch_name + '-charm'
        elif project_name == 'canonical-identity-provider':
            project_name += '-' + branch_name
        elif project_name == 'ubuntuone-servers' and branch_name == 'wheels':
            project_name += '-' + branch_name
        elif branch_name == 'dependencies':
            project_name += '-dependencies'
    elif name.startswith('lp:'):
        project_name = name[3:]
    else:
        project_name = name
    # Some more snow flakes
    if project_name == 'canonical-sentry/raven':
        project_name = 'canonical-sentry-raven'
    elif project_name == 'click-appstore/api':
        project_name = 'click-appstore-api'
    elif project_name == 'click-appstore/common':
        project_name = 'click-appstore-common'
    elif project_name == 'click-appstore/download-cache':
        project_name = 'click-appstore-download-cache'
    elif project_name == 'click-appstore/web':
        project_name = 'click-appstore-web'
    if project_name in projects:
        projects[project_name].append(name)
    else:
        projects[project_name] = [name]
    if '/' in project_name:
        print('Illegal project name: {}'.format(project_name))
    return project_name


def update_precise_job(name, attrs):
    # print('Precise job: {}'.format(name))
    pass


def update_vivid_job(name, attrs):
    # print('Vivid job: {}'.format(name))
    pass


# Both python and jenkins jobs collide on using {} to mark variable references
# To avoid the issue we use [] instead of {} in the jenkins jobs templates used
# in the *Job classes below.
class Job(object):

    def __init__(self, name, project_name, description=None, release=None,
                 irc_channel=None):
        self.name = name  # in puppet
        self.project_name = project_name  # in jenkaas
        self.description = description
        self.release = release
        self.irc_channel = irc_channel

    def expand_job_def(self):
        job_def = self.head_job_def()
        job_def += self.jobs_job_def()
        job_def = self.cleanup(job_def)
        return job_def

    def expand(self, s):
        return s.format(**self.__dict__)

    def head_job_def(self):
        head = self.expand('''- project:
    defaults: project
    name: '{project_name}'
    container: 'u1-trusty'
    slave: !include: slaves.yaml
''')
        return head

    def cleanup(self, job_def):
        job_def = job_def.replace('[', '{')
        job_def = job_def.replace(']', '}')
        return job_def


class LandingJob(Job):

    def __init__(self, name, project_name, description=None, release=None,
                 irc_channel=None,
                 verify_command=None, tree_dir=None,
                 allowed_contributors=None,
                 concurrency=None,
                 projects_to_trigger=None):
        super().__init__(name, project_name, description, release, irc_channel)
        self.verify_command = verify_command
        self.tree_dir = tree_dir
        self.allowed_contributors = allowed_contributors
        self.concurrency = concurrency
        self.projects_to_trigger = projects_to_trigger

    def head_job_def(self):
        head = super().head_job_def()
        head += self.expand('''\
    trunk: '{name}'
''')
        if self.verify_command:
            head += self.expand('''\
    landing-tests: '{verify_command}'
''')
        return head

    def jobs_job_def(self):
        return '''\
    jobs:
      - '[name]-setup-[slave]'
      - '[name]-landing-tests'
      - '[name]'
'''


class PeriodicJob(Job):

    def __init__(self, name, project_name, description=None, release=None,
                 irc_channel=None,
                 timer_spec=None, verify_command=None,
                 artifacts=None, results_file=None,
                 branch_url=None, notify_email=None):
        super().__init__(name, project_name, description, release, irc_channel)
        self.timer_spec = timer_spec
        self.verify_command = verify_command
        self.artifacts = artifacts
        self.results_file = results_file
        self.branch_url = branch_url
        self.notify_email = notify_email

    def jobs_job_def(self):
        return '''\
    jobs:
      - '[name]-tests'
'''


job_classes = {
    'jenkins::cron::tests': PeriodicJob,
    'jenkins::tarmac::job': LandingJob,
}


def update_trusty_job(name, attrs):
    project_name = project_name_from_puppet_name(name)
    print('Project: {} Trusty job: {}'.format(project_name, name))
    job = job_classes[attrs.pop('type')](name, project_name, **attrs)
    with open(os.path.join(jenkaas_job_dir,
                           '{}.yaml'.format(project_name)), 'w') as job_file:
        job_file.write(job.expand_job_def())


migrated_jobs = [
    'lp:canonical-identity-provider',
    'lp:canonical-payment-service',
    'lp:isitdeployable',
    'lp:isitdeployable-dependencies',
    'lp:snap-realtime',
    'lp:software-center-agent',
    'lp:~ubuntuone-pqm-team/canonical-identity-provider/dependencies',
    'lp:~ubuntuone-pqm-team/canonical-payment-service/dependencies',
    'lp:~ubuntuone-pqm-team/software-center-agent/dependencies',
    # FIXME: Needs migration after checking with wes ? May be just name it
    # conn-check-charm ? -- 2016-09-28
    'lp:~ubuntuone-hackers/charms/trusty/conn-check/trunk',
    # FIXME: This has not been migrated but breaks job generation for a single
    # snow flake. -- vila 2016-0930
    'conn-check-postmerge',
    # FIXME: Not migrated, requires a better support of
    # jenkins::cron::job/PeriodicJob (or just downstream builds ?)
    # -- vila 2016-10-04
    'canonical-identity-provider-acceptance-dev',
    'canonical-identity-provider-acceptance-production',
    'canonical-identity-provider-acceptance-staging',
    # Haha. No, you won't migrate ;)
    'lp:~ubuntuone-hackers/ubuntuone-infrastructure/puppet',
    # Dead projects
    'lp:pkgme-service',  # Last run on Jul 16, 2015 3:14 PM
    # Last run on Jun 30, 2015 12:03 PM
    'lp:~ubuntuone-pqm-team/canonical-is-charms/pkgme-service',
    'lp:requestplus',  # Last run on Sep 10, 2015 3:00 PM
]

Job_attrs = dict()


def classify_attrs(attrs):
    attrs = attrs.copy()  # We will delete some while classifying
    job_type = attrs.pop('type')
    if job_type not in Job_attrs:
        Job_attrs[job_type] = dict()
    this_type = Job_attrs[job_type]
    for attr in attrs.keys():
        if attr not in this_type:
            this_type[attr] = 0
        this_type[attr] += 1


def update_jobs(jobs):
    for name, attrs in jobs.items():
        if name in migrated_jobs:
            # Don't attempt to process already migrated jobs
            continue
        if 'assigned_node' in attrs:
            assigned_nodes = attrs.pop('assigned_node')
            attrs['release'] = release = assigned_nodes.split('&&')[-1]
        classify_attrs(attrs)
        if name == 'lp:click-toolbelt':
            update_trusty_job(name, attrs)
        elif 'release' not in attrs:
            # master runs precise
            update_precise_job(name, attrs)
        elif release == 'precise':
            update_precise_job(name, attrs)
        elif release == 'vivid':
            update_vivid_job(name, attrs)
        elif release == 'trusty':
            update_trusty_job(name, attrs)
        else:
            # Hu ho, what's that ?
            print('Exotic job: {} {}'.format(name, attrs))


def main(args):
    if len(args) == 2:
        global jobdefs_path
        jobdefs_path = args[1]
    jobs = collect_jobs()
    update_jobs(jobs)
    for project_name, names in projects.items():
        if len(names) > 1:
            print('Duplicate names for {}: {}'.format(project_name, names))


if __name__ == '__main__':
    main(sys.argv)
