# This file is part of Online Services Jenkaas.
#
# Copyright 2016 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals


import argparse
import logging
import sys


from olsconfig import registries


from olsjenkaas import (
    config,
    launchpad,
)


logger = logging.getLogger(__name__)


def setup_logging(level=None):
    if level is None:
        level = logging.ERROR
    if level == logging.DEBUG:
        log_format = '%(asctime)s %(name)s %(levelname)s %(message)s'
    else:
        log_format = '%(asctime)s %(levelname)s %(message)s'
    logging.basicConfig(level=level, format=log_format)


class ArgParser(argparse.ArgumentParser):
    """An argument parser for the ols-jenkaas script."""

    def __init__(self, name, description):
        script_name = 'ols-jenkaas'
        super(ArgParser, self).__init__(
            prog='{} {}'.format(script_name, name),
            description=description)

    def parse_args(self, args=None, out=None, err=None):
        """Parse arguments, overridding stdout/stderr if provided.

        :params args: Defaults to sys.argv[1:].

        :param out: Defaults to sys.stdout.

        :param err: Defaults to sys.stderr.
        """
        out_orig = sys.stdout
        err_orig = sys.stderr
        try:
            if out is not None:
                sys.stdout = out
            if err is not None:
                sys.stderr = err
            return super(ArgParser, self).parse_args(args)
        finally:
            sys.stdout = out_orig
            sys.stderr = err_orig


class CommandRegistry(registries.Registry):
    """A registry specialized for commands."""

    def register(self, cmd):
        super(CommandRegistry, self).register(
            cmd.name, cmd, help_string=cmd.description)


# All commands are registered here, defining what run() supports
command_registry = CommandRegistry()


class Command(object):

    name = 'Must be set by daughter classes'
    description = 'Must be set by daughter classes'

    def __init__(self, out=None, err=None):
        """Command constructor.

        :param out: A stream for command output.

        :param err: A stream for command errors.
        """
        if out is None:
            out = sys.stdout
        if err is None:
            err = sys.stderr
        self.out = out
        self.err = err
        self.parser = ArgParser(self.name, self.description)

    def parse_args(self, args):
        self.options = self.parser.parse_args(args, self.out, self.err)
        return self.options


class Help(Command):

    name = 'help'
    description = 'Describe ols-jenkaas commands.'

    def __init__(self, **kwargs):
        super(Help, self).__init__(**kwargs)
        self.parser.add_argument(
            'commands', metavar='COMMAND', nargs='*',
            help='Display help for each command'
            ' (All if none is given).')

    def run(self):
        if not self.options.commands:
            self.out.write('Available commands:\n')
            for cmd_name in command_registry.keys():
                cmd_class = command_registry.get(cmd_name)
                self.out.write('\t{}: {}\n'.format(cmd_class.name,
                                                   cmd_class.description))
            return
        for cmd_name in self.options.commands:
            try:
                cmd_class = command_registry.get(cmd_name)
            except KeyError:
                cmd_class = None
            if cmd_class:
                cmd = cmd_class()
                cmd.parser.print_help(self.out)
            else:
                msg = '"{}" is not a known command\n'.format(cmd_name)
                self.err.write(msg)
        return 0


command_registry.register(Help)


class ApprovedBzrProposals(Command):

    name = 'approved-bzr-proposals'
    description = 'Check for approved bzr merge proposals.'

    def __init__(self, **kwargs):
        super(ApprovedBzrProposals, self).__init__(**kwargs)
        self.parser.add_argument(
            'target', metavar='TARGET',
            help='The bazaar target branch for the merge proposals.')

    def run(self):
        self.lp = launchpad.Launchpad(config.JenkaasStack())
        target_branch = self.lp.get_bzr_branch(self.options.target)
        if target_branch is None:
            logger.error('{} is not a bzr branch'.format(
                self.options.target))
            return 1
        logger.info('Searching proposals for {}'.format(
            target_branch.web_link))
        for proposal in target_branch.landing_candidates:
            logger.info('Looking at {}'.format(proposal.web_link))
            unique_name = proposal.target_branch.unique_name
            if unique_name != target_branch.unique_name:
                logger.info('Ignoring, target is {}'.format(
                    unique_name))
                continue
            queue_status = proposal.queue_status
            if queue_status != 'Approved':
                logger.info('Ignoring, status is {}'.format(queue_status))
                continue
            # The proposal is approved
            logger.info('{} is approved'.format(proposal.web_link))
            return 0
        # None of the proposals are approved
        logger.error('No proposal is approved')
        return 1


command_registry.register(ApprovedBzrProposals)


class ApprovedGitProposals(Command):

    name = 'approved-git-proposals'
    description = 'Check for approved git merge proposals.'

    def __init__(self, **kwargs):
        super(ApprovedGitProposals, self).__init__(**kwargs)
        # FIXME: Shouldn't we mention the branch inside the repository there ?
        # Even if defaulting to 'master' -- vila 2016-12-07
        self.parser.add_argument(
            'target', metavar='TARGET',
            help='The target git repository for the merge proposals.')

    def run(self):
        self.lp = launchpad.Launchpad(config.JenkaasStack())
        target_git_repo = self.lp.get_git_repo(self.options.target)
        if target_git_repo is None:
            logger.error('{} is not a git repository'.format(
                self.options.target))
            return 1
        for proposal in target_git_repo.landing_candidates:
            logger.info('Looking at {}'.format(proposal.web_link))
            unique_name = proposal.target_git_repository.unique_name
            if unique_name != target_git_repo.unique_name:
                logger.info('Ignoring, target is {}'.format(
                    unique_name))
                continue
            queue_status = proposal.queue_status
            if queue_status != 'Approved':
                logger.info('Ignoring, status is {}'.format(queue_status))
                continue
            # The proposal is approved
            logger.info('{} is approved'.format(proposal.web_link))
            return 0
        # None of the Proposals are approved
        logger.error('No proposal is approved')
        return 1


command_registry.register(ApprovedGitProposals)


def run(args=None, out=None, err=None):
    if args is None:
        args = sys.argv[1:]
    if not args:
        cmd = Help()
        cmd_name = 'help'
    else:
        cmd_name = args[0]
        args = args[1:]
        try:
            cmd_class = command_registry.get(cmd_name)
            cmd = cmd_class(out=out, err=err)
        except KeyError:
            cmd = Help(out=out, err=err)
            args = [cmd_name]
    try:
        cmd.parse_args(args)
        return cmd.run()
    except Exception as e:
        logger.debug('{} failed'.format(cmd_name), exc_info=True)
        logger.error('{} failed: {!r}'.format(cmd_name, e))
        return -1
