# This file is part of Online Services Jenkaas.
#
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals

import logging
import os


from olstests import fixtures
from olsvms import subprocesses


from olsjenkaas import config
from olsjenkaas.tests import features


# Useful shorcut
build_tree = fixtures.build_tree


def isolate_from_disk(test):
    """Provide an isolated disk-based environment.

    A $HOME directory is setup so tests can setup arbitrary files including
    config ones.
    """
    fixtures.set_uniq_cwd(test)
    fixtures.isolate_from_env(test)
    # Isolate tests from the user environment
    test.home_dir = os.path.join(test.uniq_dir, 'home')
    os.mkdir(test.home_dir)
    os.environ['HOME'] = test.home_dir


def use_qastaging(test):
    # Get original path before isolation
    credentials_path = features.qastaging_token.path
    isolate_from_disk(test)
    # Setup to use qastaging launchpad API and friends
    conf = config.JenkaasStack()
    conf.set('launchpad.service_root', 'qastaging')
    conf.set('launchpad.credentials', '~/.config/ols-jenkaas/qastaging')
    conf.store.save_changes()
    with open(credentials_path) as fin:
        with open(conf.get('launchpad.credentials'), 'w') as fout:
            fout.write(fin.read())


def override_logging(test, stream_out, level=logging.INFO, fmt='%(message)s'):
    """Setup a logging handler, restoring the actual handlers after the test.

    This assumes a logging setup where handlers are added to the root logger
    only.

    :param stream_out: A stream where log messges will be recorded.

    :param level: The logging level to be active during the test.

    :param fmt: The logging format to use during the test.

    """
    root_logger = logging.getLogger(None)
    # Using reversed() below ensures we can modify root_logger.handlers as well
    # as providing the handlers in the right order for cleanups.
    for handler in reversed(root_logger.handlers):
        root_logger.removeHandler(handler)
        test.addCleanup(root_logger.addHandler, handler)
    # Install the new handler
    new_handler = logging.StreamHandler(stream_out)
    test.addCleanup(root_logger.removeHandler, new_handler)
    root_logger.addHandler(new_handler)
    # Install the new level, restoring the actual one after the test
    test.addCleanup(root_logger.setLevel, root_logger.level)
    root_logger.setLevel(level)


class ThatDir(object):

    def __init__(self, new_dir):
        self.new_dir = new_dir
        self.old_dir = None

    def __enter__(self):
        self.old_dir = os.getcwd()
        os.chdir(self.new_dir)
        return self

    def __exit__(self, *_):
        os.chdir(self.old_dir)


def set_bzr_identity(test, identity):
    """Set the bzr whoami."""
    test.assertTrue(test.uniq_dir in os.environ['HOME'])
    subprocesses.run(['bzr', 'whoami', identity])


def set_bzr_launchpad_access(test, identity):
    """Set the bzr launchpad login."""
    test.assertTrue(test.uniq_dir in os.environ['HOME'])
    # All tests run against qastaging
    plugin_dir = os.path.join(test.uniq_dir, 'home', '.bazaar', 'plugins')
    os.makedirs(plugin_dir)
    with open(os.path.join(plugin_dir, 'qastagingonly.py'), 'w') as f:
        f.write('from bzrlib.plugins.launchpad import lp_directory\n')
        f.write('lp_directory.LaunchpadService.DEFAULT_INSTANCE = "qastaging"')
    subprocesses.run(['bzr', 'launchpad-login', identity])
    # bazaar.qastaging.launchpad.net must be in ~/.ssh/known_hosts


def set_git_identity(test, name, email):
    """Set the git user name and email."""
    test.assertTrue(test.uniq_dir in os.environ['HOME'])
    subprocesses.run(['git', 'config', '--global', 'user.name', name])
    subprocesses.run(['git', 'config', '--global', 'user.email', email])


def set_git_launchpad_access(test, identity):
    """Configure git to access launchpad."""
    test.assertTrue(test.uniq_dir in os.environ['HOME'])
    # All tests run against qastaging
    # git.qastaging.paddev.net must be in ~/.ssh/known_hosts
    subprocesses.run(
        ['git', 'config', '--global',
         'url.ssh://{}@git.qastaging.paddev.net/.insteadOf'.format(identity),
         'lp:'])
    subprocesses.run(
        ['git', 'config', '--global', 'push.default', 'simple'])
