#
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals


import os
import unittest


from olsjenkaas import tests
from olsjenkaas.tests import (
    features,
    fixtures,
)
from olsjenkaas.tests.branches import (
    bzr,
    git,
)


@features.requires(features.bzr_identity)
class TestLocalBzrBranch(unittest.TestCase):

    def setUp(self):
        super(TestLocalBzrBranch, self).setUp()
        fixtures.isolate_from_disk(self)
        fixtures.set_bzr_identity(self, features.bzr_identity.identity)

    def test_delete(self):
        b = bzr.Branch('branch')
        b.create()
        self.assertTrue(os.path.exists('branch/.bzr'))
        b.delete()
        self.assertFalse(os.path.exists('branch'))

    def test_create(self):
        b = bzr.Branch('branch')
        self.addCleanup(b.delete)
        b.create()
        self.assertEqual('', b.status())

    def test_update(self):
        b = bzr.Branch('branch')
        self.addCleanup(b.delete)
        b.create()
        b.update('Something', '''
file: foo
content
''')
        self.assertEqual('', b.status())
        self.assertTrue(os.path.exists('branch/foo'))


@features.requires(features.git_identity)
class TestLocalGitRepo(unittest.TestCase):

    def setUp(self):
        super(TestLocalGitRepo, self).setUp()
        fixtures.isolate_from_disk(self)
        fixtures.set_git_identity(self, features.git_identity.name,
                                  features.git_identity.email)

    def test_delete(self):
        r = git.Repository('branch')
        r.create()
        self.assertTrue(os.path.exists('branch/.git'))
        r.delete()
        self.assertFalse(os.path.exists('branch'))

    def test_create(self):
        r = git.Repository('branch')
        self.addCleanup(r.delete)
        r.create()
        self.assertEqual('', r.status())

    def test_update(self):
        r = git.Repository('branch')
        self.addCleanup(r.delete)
        r.create()
        r.update('Something', '''
file: foo
content
''')
        self.assertEqual('', r.status())
        self.assertTrue(os.path.exists('branch/foo'))


class TestLocalBzrBranchWithRemote(tests.TestQAStagingWithBzr):

    def test_create_from(self):
        b = bzr.Branch('branch')
        b.create_from('lp:~ubuntuone-hackers/ols-jenkaas-test-project/trunk')

    def test_push(self):
        b = bzr.Branch('branch')
        self.addCleanup(b.delete)
        b.create()
        b.update('Something', '''
file: foo
content
''')
        lp_name = '~{}/ols-jenkaas-test-project/push-{}'.format(
            features.launchpad_identity.identity,
            os.getpid())
        url = 'lp:' + lp_name
        b.push(url)
        lp_br = self.lp.get_bzr_branch(url)
        self.assertIsNot(None, lp_br)
        self.addCleanup(lp_br.lp_delete)
        self.assertEqual(lp_name, lp_br.unique_name)


class TestLocalGitRepoWithRemote(tests.TestQAStagingWithGit):

    def test_create_from(self):
        r = git.Repository('branch')
        r.create_from('lp:ols-jenkaas-test-project')

    def test_push(self):
        r = git.Repository('branch')
        self.addCleanup(r.delete)
        r.create()
        r.update('Something', '''
file: foo
content
''')
        lp_name = ('~{}/ols-jenkaas-test-project/+git/push-{}'.format(
            features.launchpad_identity.identity,
            os.getpid()))
        url = 'lp:' + lp_name
        r.push(url)
        lp_br = self.lp.get_git_repo(url)
        self.assertIsNot(None, lp_br)
        self.addCleanup(lp_br.lp_delete)
        self.assertEqual(lp_name, lp_br.unique_name)
