# This file is part of Online Services Jenkaas.
#
# Copyright 2016, 2017 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals


import io
import unittest


from olstests import assertions


from olsjenkaas import (
    commands,
    tests,
)
from olsjenkaas.tests import (
    fixtures,
)


class TestHelpOptions(unittest.TestCase):

    def setUp(self):
        super(TestHelpOptions, self).setUp()
        self.out = io.StringIO()
        self.err = io.StringIO()

    def parse_args(self, args):
        help_cmd = commands.Help(out=self.out, err=self.err)
        return help_cmd.parse_args(args)

    def test_defaults(self):
        ns = self.parse_args([])
        self.assertEqual([], ns.commands)

    def test_single_command(self):
        ns = self.parse_args(['setup'])
        self.assertEqual(['setup'], ns.commands)

    def test_several_commands(self):
        ns = self.parse_args(['help', 'setup', 'not-a-command'])
        self.assertEqual(['help', 'setup', 'not-a-command'], ns.commands)


class TestHelp(unittest.TestCase):

    def setUp(self):
        super(TestHelp, self).setUp()
        self.out = io.StringIO()
        self.err = io.StringIO()

    def assertHelp(self, expected, args=None):
        if args is None:
            args = []
        help_cmd = commands.Help(out=self.out, err=self.err)
        help_cmd.parse_args(args)
        help_cmd.run()
        assertions.assertMultiLineAlmostEqual(self, expected,
                                              self.out.getvalue())

    def test_help_alone(self):
        self.assertHelp('''\
Available commands:
\tapproved-bzr-proposals: Check for approved bzr merge proposals.
\tapproved-git-proposals: Check for approved git merge proposals.
\thelp: Describe ols-jenkaas commands.
''')

    def test_help_help(self):
        self.assertHelp('''\
usage: ols-jenkaas... help [-h] [COMMAND [COMMAND ...]]

Describe ols-jenkaas commands.

positional arguments:
  COMMAND     Display help for each command (All if none is given).

optional arguments:
  -h, --help  show this help message and exit
''',
                        ['help'])


class TestApprovedGitProposalsOptions(unittest.TestCase):

    def setUp(self):
        super(TestApprovedGitProposalsOptions, self).setUp()
        self.out = io.StringIO()
        self.err = io.StringIO()

    def parse_args(self, args):
        agp_cmd = commands.ApprovedGitProposals(out=self.out, err=self.err)
        return agp_cmd.parse_args(args)

    def test_nothing(self):
        with self.assertRaises(SystemExit):
            self.parse_args([])

    def test_too_much(self):
        with self.assertRaises(SystemExit):
            self.parse_args(['foo', 'bar'])

    def test_target(self):
        ns = self.parse_args(['bar'])
        self.assertEqual('bar', ns.target)


class TestApprovedGitProposals(tests.TestAgainstLaunchpadQAStaging):

    def setUp(self):
        super(TestApprovedGitProposals, self).setUp()
        self.out = io.StringIO()
        self.err = io.StringIO()
        self.log_stream = io.StringIO()
        fixtures.override_logging(self, self.log_stream)

    def run_agp(self, args):
        agp_cmd = commands.ApprovedGitProposals(out=self.out, err=self.err)
        agp_cmd.parse_args(args)
        return agp_cmd.run()

    def test_unknown_branch(self):
        ret = self.run_agp(['lp:I-dont-exist'])
        self.assertEqual(1, ret)
        self.assertEqual('lp:I-dont-exist is not a git repository\n',
                         self.log_stream.getvalue())

    def test_no_proposals(self):
        ret = self.run_agp(['lp:ols-jenkaas-test-project'])
        self.assertEqual(1, ret)


# FIXME: We need more infra to:
# - setup a repo with [no] approved MPS

# MISSINGTEST: {unknwon repo, not a git repo, correct repo}
# MISSINGTEST: [no [approved] Proposals


class TestApprovedBzrProposalsOptions(unittest.TestCase):

    def setUp(self):
        super(TestApprovedBzrProposalsOptions, self).setUp()
        self.out = io.StringIO()
        self.err = io.StringIO()

    def parse_args(self, args):
        abp_cmd = commands.ApprovedBzrProposals(out=self.out, err=self.err)
        return abp_cmd.parse_args(args)

    def test_nothing(self):
        with self.assertRaises(SystemExit):
            self.parse_args([])

    def test_too_much(self):
        with self.assertRaises(SystemExit):
            self.parse_args(['foo', 'bar'])

    def test_target(self):
        ns = self.parse_args(['bar'])
        self.assertEqual('bar', ns.target)


class TestApprovedBzrProposals(tests.TestAgainstLaunchpadQAStaging):

    def setUp(self):
        super(TestApprovedBzrProposals, self).setUp()
        self.out = io.StringIO()
        self.err = io.StringIO()
        self.log_stream = io.StringIO()
        fixtures.override_logging(self, self.log_stream)

    def run_abp(self, args):
        abp_cmd = commands.ApprovedBzrProposals(out=self.out, err=self.err)
        abp_cmd.parse_args(args)
        return abp_cmd.run()

    def test_unknown_branch(self):
        ret = self.run_abp(['lp:I-dont-exist'])
        self.assertEqual(1, ret)
        self.assertEqual('lp:I-dont-exist is not a bzr branch\n',
                         self.log_stream.getvalue())

    def test_no_proposals(self):
        ret = self.run_abp(['lp:ols-jenkaas-test-project'])
        self.assertEqual(1, ret)

# FIXME: We need more infra to:
# - setup a branch with [no] approved MPS

# MISSINGTEST: {unknwon branch, not a branch, correct branch}
# MISSINGTEST: [no [approved]] Proposals
